use log::{info, trace};
use sieverman::ConnectionInfo;

use crate::protocol::BackToFront;

pub(crate) async fn run(
    to_frontend_tx: tokio::sync::mpsc::Sender<BackToFront>,
) -> anyhow::Result<()> {
    trace!("Starting up sieverman…");
    let info = ConnectionInfo::new("teilgedanken.de", 4190);
    let mut connected = info.connect().await.unwrap();
    info!("connected to the server");
    loop {
        match connected.read_introduction().await {
            Ok(sieverman::IsComplete::Yes(c)) => {
                connected = c;
                break;
            }
            Ok(sieverman::IsComplete::No(c)) => {
                trace!("Incomplete data waiting for some time…");
                connected = c
            }
            Err(e) => return Err(anyhow::anyhow!("something went wrong: {}", e)),
        };
        tokio::time::sleep(std::time::Duration::from_millis(100)).await;
        connected.log_buffer().await;
    }
    trace!("Fully read the introduction:");
    connected.log_buffer().await;
    connected.log_server_settings().await;
    to_frontend_tx
        .send(BackToFront::ServerConnected(
            connected.get_greeting().unwrap(),
        ))
        .await
        .unwrap();
    Ok(())
}
