use std::{cell::RefCell, rc::Rc};

use gtk4::{
    glib,
    prelude::{ApplicationExt, BoxExt, GtkWindowExt, WidgetExt},
    Application, ApplicationWindow, Builder, Label,
};
use log::trace;

use crate::protocol::BackToFront;

pub(crate) fn get_app(
    from_backend_rx: Rc<RefCell<Option<tokio::sync::mpsc::Receiver<BackToFront>>>>,
) -> Application {
    let app = Application::builder()
        .application_id("de.teilgedanken.sieverman")
        .build();

    app.connect_activate(move |app| build_ui(app, from_backend_rx.clone()));

    app
}

fn build_ui(
    app: &gtk4::Application,
    from_backend_rx: Rc<RefCell<Option<tokio::sync::mpsc::Receiver<BackToFront>>>>,
) {
    let action_content_status = gtk4::Box::builder()
        .orientation(gtk4::Orientation::Vertical)
        .spacing(5)
        .build();
    let action = gtk4::ActionBar::new();
    let status = gtk4::Statusbar::new();
    let main_log_server_info = gtk4::Box::builder()
        .orientation(gtk4::Orientation::Horizontal)
        .hexpand(true)
        .vexpand(true)
        .halign(gtk4::Align::Fill)
        .build();
    action_content_status.append(&action);
    action_content_status.append(&main_log_server_info);
    action_content_status.append(&status);
    let log = gtk4::Label::new(Some("Log"));
    let server_info = gtk4::Label::new(Some("Server Info"));
    main_log_server_info.append(&log);
    main_log_server_info.append(&server_info);
    let main_window = gtk4::ApplicationWindow::builder()
        .application(app)
        .title("Sieverman")
        .height_request(400)
        .width_request(400)
        .child(&action_content_status)
        .build();
    main_window.present();

    let server_info = Rc::new(RefCell::new(server_info));

    let future = {
        let mut data_event_receiver = from_backend_rx.take().expect("data_event_reciver");
        async move {
            while let Some(event) = data_event_receiver.recv().await {
                trace!("data event: {:?}", event);
                match event {
                    BackToFront::ServerConnected(message) => {
                        trace!("Received {}", message);
                        server_info.borrow_mut().set_text(&message);
                    }
                }
            }
        }
    };

    let c = glib::MainContext::default();
    c.spawn_local(future);
    main_window.present();
}
